@@ -20,10 +20,11 @@ require File.expand_path('../../test_helper', __FILE__)
 class RepositoryGitTest < ActiveSupport::TestCase
   fixtures :projects, :repositories, :enabled_modules, :users, :roles
 
-  # No '..' in the repository path
-  REPOSITORY_PATH = RAILS_ROOT.gsub(%r{config\/\.\.}, '') + '/tmp/test/git_repository'
+  REPOSITORY_PATH = Rails.root.join('tmp/test/git_repository').to_s
   REPOSITORY_PATH.gsub!(/\//, "\\") if Redmine::Platform.mswin?
 
+  NUM_REV = 21
+
   FELIX_HEX  = "Felix Sch\xC3\xA4fer"
   CHAR_1_HEX = "\xc3\x9c"
 
@@ -65,10 +66,11 @@ class RepositoryGitTest < ActiveSupport::TestCase
     def test_fetch_changesets_from_scratch
       assert_nil @repository.extra_info
 
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
 
-      assert_equal 21, @repository.changesets.count
+      assert_equal NUM_REV, @repository.changesets.count
       assert_equal 33, @repository.changes.count
 
       commit = @repository.changesets.find(:first, :order => 'committed_on ASC')
@@ -89,9 +91,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_fetch_changesets_incremental
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
-      assert_equal 21, @repository.changesets.count
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       assert_equal 33, @repository.changes.count
       extra_info_db = @repository.extra_info["branches"]
       assert_equal 4, extra_info_db.size
@@ -111,7 +114,7 @@ class RepositoryGitTest < ActiveSupport::TestCase
       @repository.changesets.each do |rev|
         rev.destroy if del_revs.detect {|r| r == rev.scmid.to_s }
       end
-      @repository.reload
+      @project.reload
       cs1 = @repository.changesets
       assert_equal 15, cs1.count
       h = @repository.extra_info.dup
@@ -119,20 +122,21 @@ class RepositoryGitTest < ActiveSupport::TestCase
             "4a07fe31bffcf2888791f3e6cbc9c4545cefe3e8"
       @repository.merge_extra_info(h)
       @repository.save
-      @repository.reload
+      @project.reload
       extra_info_db_1 = @repository.extra_info["branches"]
       assert_equal "4a07fe31bffcf2888791f3e6cbc9c4545cefe3e8",
                     extra_info_db_1["master"]["last_scmid"]
 
       @repository.fetch_changesets
-      assert_equal 21, @repository.changesets.count
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
     end
 
     def test_fetch_changesets_invalid_rev
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
-      assert_equal 21, @repository.changesets.count
-      assert_equal 33, @repository.changes.count
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       extra_info_db = @repository.extra_info["branches"]
       assert_equal 4, extra_info_db.size
       assert_equal "1ca7f5ed374f3cb31a93ae5215c2e25cc6ec5127",
@@ -151,7 +155,7 @@ class RepositoryGitTest < ActiveSupport::TestCase
       @repository.changesets.each do |rev|
         rev.destroy if del_revs.detect {|r| r == rev.scmid.to_s }
       end
-      @repository.reload
+      @project.reload
       cs1 = @repository.changesets
       assert_equal 15, cs1.count
       h = @repository.extra_info.dup
@@ -159,34 +163,55 @@ class RepositoryGitTest < ActiveSupport::TestCase
             "abcd1234efgh"
       @repository.merge_extra_info(h)
       @repository.save
-      @repository.reload
+      @project.reload
       extra_info_db_1 = @repository.extra_info["branches"]
       assert_equal "abcd1234efgh",
                     extra_info_db_1["master"]["last_scmid"]
 
       @repository.fetch_changesets
+      @project.reload
       assert_equal 15, @repository.changesets.count
     end
 
+    def test_parents
+      assert_equal 0, @repository.changesets.count
+      @repository.fetch_changesets
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
+      r1 = @repository.find_changeset_by_name("7234cb2750b63")
+      assert_equal [], r1.parents
+      r2 = @repository.find_changeset_by_name("899a15dba03a3")
+      assert_equal 1, r2.parents.length
+      assert_equal "7234cb2750b63f47bff735edc50a1c0a433c2518",
+                   r2.parents[0].identifier
+      r3 = @repository.find_changeset_by_name("32ae898b720c2")
+      assert_equal 2, r3.parents.length
+      r4 = [r3.parents[0].identifier, r3.parents[1].identifier].sort
+      assert_equal "4a07fe31bffcf2888791f3e6cbc9c4545cefe3e8", r4[0]
+      assert_equal "7e61ac704deecde634b51e59daa8110435dcb3da", r4[1]
+    end
+
     def test_db_consistent_ordering_init
       assert_nil @repository.extra_info
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
       assert_equal 1, @repository.extra_info["db_consistent"]["ordering"]
     end
 
     def test_db_consistent_ordering_before_1_2
       assert_nil @repository.extra_info
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
-      assert_equal 21, @repository.changesets.count
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       assert_not_nil @repository.extra_info
       @repository.write_attribute(:extra_info, nil)
       @repository.save
       assert_nil @repository.extra_info
-      assert_equal 21, @repository.changesets.count
+      assert_equal NUM_REV, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
       assert_equal 0, @repository.extra_info["db_consistent"]["ordering"]
 
       del_revs = [
@@ -200,7 +225,7 @@ class RepositoryGitTest < ActiveSupport::TestCase
       @repository.changesets.each do |rev|
         rev.destroy if del_revs.detect {|r| r == rev.scmid.to_s }
       end
-      @repository.reload
+      @project.reload
       cs1 = @repository.changesets
       assert_equal 15, cs1.count
       assert_equal 0, @repository.extra_info["db_consistent"]["ordering"]
@@ -209,19 +234,21 @@ class RepositoryGitTest < ActiveSupport::TestCase
             "4a07fe31bffcf2888791f3e6cbc9c4545cefe3e8"
       @repository.merge_extra_info(h)
       @repository.save
-      @repository.reload
+      @project.reload
       extra_info_db_1 = @repository.extra_info["branches"]
       assert_equal "4a07fe31bffcf2888791f3e6cbc9c4545cefe3e8",
                     extra_info_db_1["master"]["last_scmid"]
 
       @repository.fetch_changesets
-      assert_equal 21, @repository.changesets.count
+      assert_equal NUM_REV, @repository.changesets.count
       assert_equal 0, @repository.extra_info["db_consistent"]["ordering"]
     end
 
     def test_latest_changesets
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       # with limit
       changesets = @repository.latest_changesets('', nil, 2)
       assert_equal 2, changesets.size
@@ -341,8 +368,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
       elsif JRUBY_SKIP
         puts JRUBY_SKIP_STR
       else
+        assert_equal 0, @repository.changesets.count
         @repository.fetch_changesets
-        @repository.reload
+        @project.reload
+        assert_equal NUM_REV, @repository.changesets.count
         changesets = @repository.latest_changesets(
                     "latin-1-dir/test-#{@char_1}-subdir", '1ca7f5ed')
         assert_equal [
@@ -352,8 +381,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_find_changeset_by_name
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       ['7234cb2750b63f47bff735edc50a1c0a433c2518', '7234cb2750b'].each do |r|
         assert_equal '7234cb2750b63f47bff735edc50a1c0a433c2518',
                      @repository.find_changeset_by_name(r).revision
@@ -361,24 +392,30 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_find_changeset_by_empty_name
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       ['', ' ', nil].each do |r|
         assert_nil @repository.find_changeset_by_name(r)
       end
     end
 
     def test_identifier
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       c = @repository.changesets.find_by_revision(
                           '7234cb2750b63f47bff735edc50a1c0a433c2518')
       assert_equal c.scmid, c.identifier
     end
 
     def test_format_identifier
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       c = @repository.changesets.find_by_revision(
                           '7234cb2750b63f47bff735edc50a1c0a433c2518')
       assert_equal '7234cb27', c.format_identifier
@@ -395,8 +432,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_log_utf8
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       str_felix_hex  = FELIX_HEX.dup
       if str_felix_hex.respond_to?(:force_encoding)
           str_felix_hex.force_encoding('UTF-8')
@@ -407,8 +446,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_previous
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       %w|1ca7f5ed374f3cb31a93ae5215c2e25cc6ec5127 1ca7f5ed|.each do |r1|
         changeset = @repository.find_changeset_by_name(r1)
         %w|64f1f3e89ad1cb57976ff0ad99a107012ba3481d 64f1f3e89ad1|.each do |r2|
@@ -418,8 +459,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_previous_nil
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       %w|7234cb2750b63f47bff735edc50a1c0a433c2518 7234cb2|.each do |r1|
         changeset = @repository.find_changeset_by_name(r1)
         assert_nil changeset.previous
@@ -427,8 +470,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_next
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       %w|64f1f3e89ad1cb57976ff0ad99a107012ba3481d 64f1f3e89ad1|.each do |r2|
         changeset = @repository.find_changeset_by_name(r2)
         %w|1ca7f5ed374f3cb31a93ae5215c2e25cc6ec5127 1ca7f5ed|.each do |r1|
@@ -438,8 +483,10 @@ class RepositoryGitTest < ActiveSupport::TestCase
     end
 
     def test_next_nil
+      assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
-      @repository.reload
+      @project.reload
+      assert_equal NUM_REV, @repository.changesets.count
       %w|67e7792ce20ccae2e4bb73eed09bb397819c8834 67e7792ce20cca|.each do |r1|
         changeset = @repository.find_changeset_by_name(r1)
         assert_nil changeset.next
